//*****************************************************************************
// LCMeter
// Version 1.0 Nov 2005
//
// 1.0 -> First Release
//
// Sylvain Bissonnette
//*****************************************************************************
// Editor : UltraEdit32
//*****************************************************************************
//
//                    R E T U R N   S T A C K   2 4
//                          X T A L  16 MHZ
//
//*****************************************************************************
//                          P I N   U S A G E
//
// PB0 -> n/c
// PB1 -> n/c
// PB2 -> n/c
// PB3 -> n/c
// PB4 -> n/c
// PB5 -> n/c
// PB6 -> n/c
//
// PC0 -> LCD Data bit 7
// PC1 -> LCD Data bit 6
// PC2 -> LCD Data bit 5 
// PC3 -> LCD Data bit 4
// PC4 -> LCD E
// PC5 -> LCD RS
// PC6 -> RESET
//
// PD0 -> n/c
// PD1 -> Use serial port as timer
// PD2 -> n/c
// PD3 -> LED
// PD4 -> n/c
// PD5 -> n/c
// PD6 -> n/c
//
//*****************************************************************************
//                         T I M E R   U S A G E
//
// Timer 0 not use
// Timer 1 not use
// Timer 2 not use
//
//*****************************************************************************

//*****************************************************************************
//                      I N C L U D E
//*****************************************************************************
#include <iom8v.h>
#include <shortnametype.h>
#include <macros.h>
#include <stdlib.h>
#include <stdio.h>

//*****************************************************************************
//                      D E F I N E
//*****************************************************************************
#define TRUE              1
#define FALSE             0
#define XTAL              16000000

// LCD
#define LCD_D7            0x01
#define LCD_D6            0x02
#define LCD_D5            0x04
#define LCD_D4            0x08
#define LCD_E             0x10
#define LCD_RS            0x20

#define LCD_PIN           PINC
#define LCD_DDR           DDRC
#define LCD_PORT          PORTC

#define LCD_DATA          0x01
#define LCD_CTRL          0x00

//*****************************************************************************
//                  P R O T O T Y P E
//*****************************************************************************
void main(void);

// LCD
void LCDInit(ushort x,ushort y);
void LCDClrSCR(void);
void LCDGotoXY(ushort,ushort);
void LCDWriteString(char *);
void LCDWriteConstString(flash char *);
void LCDWriteData(ushort,ushort);
void LCDWait(void);
void LCDDelay50us(int Delay);

// Serial Port
void SerialInit(void);

// Main Function
ulong MesureLC(void);
void CounterInit(void);

//*****************************************************************************
//            G L O B A L   V A R I A B L E
//*****************************************************************************
// LCD
ushort MaxX, MaxY;

// Software
ulong CountLow = 0;
ulong CountHigh = 0;
ulong Count = 0;

float BaseFrequancy;

//*****************************************************************************
//                      M A I N
//*****************************************************************************
void main(void)
{
 	uint i,j;
	char Buffer[20];
	float Frequancy;
	ulong FrequancyLong;	
	float Value;
	ulong ValueLong;
	ulong Display1,Display2;
	int x;
	
	LCDInit(20,2);
	SerialInit();
	CounterInit();
	SEI();
	
	LCDGotoXY(1,1);
	LCDWriteConstString("    LC M e t e r\0");
	LCDGotoXY(1,2);
	LCDWriteConstString(" Sylvain Bissonnette\0");
	for (i=0;i<65000;i++) for (j=0;j<5;j++) WDR();
	LCDClrSCR();
	
	LCDGotoXY(1,1);
	LCDWriteConstString("    Calibration\0");
	LCDGotoXY(1,2);
	LCDWriteConstString("    Please Wait\0");
	for (i=0;i<3;i++) BaseFrequancy = MesureLC();
	LCDClrSCR();
		
	while(TRUE)
	{	
		Frequancy = (float)MesureLC();
		Value = ((BaseFrequancy * BaseFrequancy) / (Frequancy * Frequancy))  - 1; 
		
		FrequancyLong = Frequancy;
		ValueLong = Value * 1000;
		
		Display1 = ValueLong;
		
		if (ValueLong < 1000) 
		{
			Display1 = Display1;
			csprintf(&Buffer[0],"Cap: %lu pF            \0", Display1);
		}
		else if (ValueLong < 1000000)
		{
			Display1 /= 1000;
			csprintf(&Buffer[0],"Cap: %lu nF            \0", Display1);
		}
		else if (ValueLong < 1000000000)
		{
			Display1 /= 1000000;
			csprintf(&Buffer[0],"Cap: %lu uF            \0", Display1);
		}			
		
		
		LCDGotoXY(1,2);
		LCDWriteString(&Buffer[0]);
		
		csprintf(&Buffer[0],"Freq: %lu           \0", FrequancyLong);
		LCDGotoXY(1,1);
		LCDWriteString(&Buffer[0]);
		
		for (i=0;i<65000;i++) WDR();
	}
}

/******************************************************************************

Name:         void SerialInit()

Description:  Initialize Serial Port

Input:        none

Output:       none

Misc:

******************************************************************************/
void SerialInit(void)
{
   UBRRH = ((XTAL / (8 * 250000)) - 1)>>8;	// 2400 at 16Mhz
   UBRRL = (XTAL / (8 * 250000)) - 1;        
	 UCSRA = (1<<U2X);
	 UCSRB = (1<<TXEN);
	 UCSRC = (1<<URSEL) + (1<<UCSZ1) + (1<<UCSZ0);
}

/******************************************************************************

Name:         void CounterInit()

Description:  Initialize Counter

Input:        none

Output:       none

Misc:

******************************************************************************/
void CounterInit(void)
{
	 TIMSK |= (1<<OCIE1A);
	 TCCR1B = (1<<CS12) + (1<<CS11) + (1<<CS10);
	 OCR1A = 0xffff;

	 DDRD |= (1<<PD3);
}

/******************************************************************************

Name:         ulong MesureLC()

Description:  Mesure LC Value

Input:        none

Output:       none

Misc:

******************************************************************************/
ulong MesureLC(void)
{
 int i;
 
 PORTD |= (1<<PD3);
 
 CountHigh = 0;
 TCNT1 = 0;
 
 for (i=0;i<24999;i++)
 {
  while(!(UCSRA & (1<<UDRE))) WDR();
  UDR = 0x55;
 }
 while(!(UCSRA & (1<<TXC))) WDR();
 
 CountLow = TCNT1;
 Count = CountLow + (CountHigh << 16); 

 PORTD = ~(1<<PD3);
 return Count;
}

/******************************************************************************

Name:         void HighByteCount(void)

Description:  High Byte Counter

Input:        none

Output:       none

Misc:

******************************************************************************/
#pragma interrupt_handler HighByteCount:iv_TIMER1_COMPA
void HighByteCount(void)
{
	CountHigh++;
}

//*****************************************************************************
// LCD Code
//*****************************************************************************

/******************************************************************************

Name:         void LCDClrSCR(void)

Description:  Clear the LCD

Input:        none

Output:       none

Misc:

******************************************************************************/
void LCDClrSCR()
{
  LCDWriteData(LCD_CTRL,0x01);      // Clear display
  LCDDelay50us(100);
}

/******************************************************************************

Name:         LCDGotoXY(ushort x, ushort y)

Description:  Position cursor on the LCD at X & Y location

Input:        X -> X position on the LCD
              Y -> Y position on the LCD

Output:       none

Misc:

******************************************************************************/
void LCDGotoXY(ushort x,ushort y)
{
  ushort address;

  x--;

  if (MaxY < 3)
  {
    switch(y)
    {
      case '\x01' :
      address = 0 + x;
      break;
      case '\x02' :
      address = 64 + x;
      break;
    }
  }
  else
  {
    switch(y)
    {
      case '\x01' :
      address = 0 + x;
      break;
      case '\x02' :
      address = 64 + x;
      break;
      case '\x03' :
      address = 20 + x;
      break;
      case '\x04' :
      address = 84 + x;
      break;
    }
  }
  LCDWriteData(LCD_CTRL,address | 0x80);
}

/******************************************************************************

Name:         void LCDWriteString(char *ptr)

Description:  Write a string from RAM on the LCD

Input:        string pointer

Output:       none

Misc:

******************************************************************************/
void LCDWriteString(char *ptr)
{
  ushort i;

  for (i=1;i<21;i++)
  {
    if (*ptr == 0x00) break;
    LCDWriteData(LCD_DATA,*ptr++);
  }
}

/******************************************************************************

Name:         void LCDWriteConstString(const char *ptr)

Description:  Write a constant string on the LCD

Input:        string pointer

Output:       none

Misc:

******************************************************************************/
void LCDWriteConstString(flash char *ptr)
{
  ushort i;

  for (i=1;i<21;i++)
  {
    if (*ptr == 0x00) break;
    LCDWriteData(LCD_DATA,*ptr++);
  }
}

/******************************************************************************

Name:         void LCDWriteData(ushort rs, ushort ch)

Description:  Write a byte in rs of the LCD

Input:        rs -> Register select
              ch -> byte to write

Output:       none

Misc:

******************************************************************************/
void LCDWriteData(ushort rs,ushort ch)
{
  LCDWait();

	LCD_PORT = 0x00;
  if ((ch & 0x80) == 0x80) LCD_PORT |= LCD_D7;
  if ((ch & 0x40) == 0x40) LCD_PORT |= LCD_D6;
  if ((ch & 0x20) == 0x20) LCD_PORT |= LCD_D5;
  if ((ch & 0x10) == 0x10) LCD_PORT |= LCD_D4;
  if (rs == 1) LCD_PORT |= LCD_RS;
  LCDDelay50us(1);
  LCD_PORT |= LCD_E;
  LCD_PORT &= ~LCD_E;

	LCD_PORT = 0x00;
  if ((ch & 0x08) == 0x08) LCD_PORT |= LCD_D7;
  if ((ch & 0x04) == 0x04) LCD_PORT |= LCD_D6;
  if ((ch & 0x02) == 0x02) LCD_PORT |= LCD_D5;
  if ((ch & 0x01) == 0x01) LCD_PORT |= LCD_D4;
  if (rs == 1) LCD_PORT |= LCD_RS;
  LCDDelay50us(1);
  LCD_PORT |= LCD_E;
  LCD_PORT &= ~LCD_E;
}

/******************************************************************************

Name:         void LCDWait(void)

Description:  wait for the LCD to be ready

Input:        none

Output:       none

Misc:     

******************************************************************************/
void LCDWait(void)
{
  LCDDelay50us(1);
}

/******************************************************************************

Name:         void LCDDelay50us(int Delay)

Description:  Delay of 50 us with a 16Mhz resonator

Input:        Delay X x 50us

Output:       none

Misc:

******************************************************************************/
void LCDDelay50us(int Delay)
{
  int i,j;

  for (i=0;i<Delay;i++)
  {
    for (j=1;j<(XTAL/133333);j++);
    asm("WDR");
  }
}

/******************************************************************************

Name:         void LCDInit(ushort X, ushort Y)

Description:  Initialize LCD in 4bit mode

Input:        ushort X -> X size
              ushort Y -> Y size

Output:       none

Misc:

******************************************************************************/
void LCDInit(ushort X, ushort Y)
{
  MaxX = X;
  MaxY = Y;
  LCD_DDR = LCD_D7+LCD_D6+LCD_D5+LCD_D4+LCD_E+LCD_RS;
  LCD_PORT = ~(LCD_D7+LCD_D6+LCD_D5+LCD_D4+LCD_E+LCD_RS);
  LCDDelay50us(340);

  LCD_PORT = (LCD_D5 + LCD_D4);               // Function Set 8 bit 3 time
  LCD_PORT |= LCD_E;
  LCD_PORT &= ~LCD_E;
  LCDDelay50us(100);
  LCD_PORT |= LCD_E;
  LCD_PORT &= ~LCD_E;
  LCDDelay50us(100);
  LCD_PORT |= LCD_E;
  LCD_PORT &= ~LCD_E;
  LCDDelay50us(100);

  LCD_PORT = LCD_D5;                          // Function Set 4 bit
  LCD_PORT |= LCD_E;
  LCD_PORT &= ~LCD_E;
  LCDDelay50us(100);

  if (Y == 1) LCDWriteData(LCD_CTRL,0x20);    // 1 line
  else LCDWriteData(LCD_CTRL,0x28);           // 2 line
  LCDWriteData(LCD_CTRL,0x0c);                // Disp ON-Cur OFF-Blink OFF
  LCDWriteData(LCD_CTRL,0x01);                // Clear display
  LCDDelay50us(50);
  LCDWriteData(LCD_CTRL,0x06);                // Cursor INC Shift OFF
}
